@@ -68,21 +68,22 @@ static void sender_cb               (IndicateListener * listener,
                                      IndicateListenerServer * server,
                                      IndicateListenerIndicator * indicator,
                                      gchar * property,
-                                     gchar * propertydata,
+                                     const gchar * propertydata,
                                      gpointer data);
 static void time_cb                 (IndicateListener * listener,
                                      IndicateListenerServer * server,
                                      IndicateListenerIndicator * indicator,
                                      gchar * property,
-                                     GTimeVal * propertydata,
+                                     const GTimeVal * propertydata,
                                      gpointer data);
 static void icon_cb                 (IndicateListener * listener,
                                      IndicateListenerServer * server,
                                      IndicateListenerIndicator * indicator,
                                      gchar * property,
-                                     gchar * propertydata,
+                                     const gchar * propertydata,
                                      gpointer data);
 static void activate_cb             (ImMenuItem * self,
+                                     guint timestamp,
                                      gpointer data);
 static void indicator_modified_cb   (IndicateListener * listener,
                                      IndicateListenerServer * server,
@@ -172,7 +173,7 @@ im_menu_item_finalize (GObject *object)
 /* Call back for getting icon data.  It just passes it along
    to the indicator so that it can visualize it.  Not our problem. */
 static void
-icon_cb (IndicateListener * listener, IndicateListenerServer * server, IndicateListenerIndicator * indicator, gchar * property, gchar * propertydata, gpointer data)
+icon_cb (IndicateListener * listener, IndicateListenerServer * server, IndicateListenerIndicator * indicator, gchar * property, const gchar * propertydata, gpointer data)
 {
 	dbusmenu_menuitem_property_set(DBUSMENU_MENUITEM(data), INDICATOR_MENUITEM_PROP_ICON, propertydata);
 	return;
@@ -213,6 +214,10 @@ update_time (ImMenuItem * self)
 	}
 
 	if (elapsed_minutes < 60) {
+		/* TRANSLATORS: This string is used to represent the number of minutes
+		                since an IM has occured.  It is in the right column
+		                of a menu so being brief is desirable, but one character
+		                is not a requirement. */
 		timestring = g_strdup_printf(ngettext("%d m", "%d m", elapsed_minutes), elapsed_minutes);
 	} else {
 		guint elapsed_hours = elapsed_minutes / 60;
@@ -222,6 +227,10 @@ update_time (ImMenuItem * self)
 			elapsed_hours += 1;
 		}
 
+		/* TRANSLATORS: This string is used to represent the number of hours
+		                since an IM has occured.  It is in the right column
+		                of a menu so being brief is desirable, but one character
+		                is not a requirement. */
 		timestring = g_strdup_printf(ngettext("%d h", "%d h", elapsed_hours), elapsed_hours);
 	}
 
@@ -250,7 +259,7 @@ time_update_cb (gpointer data)
    looking for.  It should only be called once at the init with a new
    indicator and again when the value changes. */
 static void
-time_cb (IndicateListener * listener, IndicateListenerServer * server, IndicateListenerIndicator * indicator, gchar * property, GTimeVal * propertydata, gpointer data)
+time_cb (IndicateListener * listener, IndicateListenerServer * server, IndicateListenerIndicator * indicator, gchar * property, const GTimeVal * propertydata, gpointer data)
 {
 	g_debug("Got Time info");
 	ImMenuItem * self = IM_MENU_ITEM(data);
@@ -282,7 +291,7 @@ time_cb (IndicateListener * listener, IndicateListenerServer * server, IndicateL
 /* Callback from libindicate that is for getting the sender information
    on a particular indicator. */
 static void
-sender_cb (IndicateListener * listener, IndicateListenerServer * server, IndicateListenerIndicator * indicator, gchar * property, gchar * propertydata, gpointer data)
+sender_cb (IndicateListener * listener, IndicateListenerServer * server, IndicateListenerIndicator * indicator, gchar * property, const gchar * propertydata, gpointer data)
 {
 	g_debug("Got Sender Information: %s", propertydata);
 	ImMenuItem * self = IM_MENU_ITEM(data);
@@ -310,7 +319,7 @@ sender_cb (IndicateListener * listener, IndicateListenerServer * server, Indicat
    that on the menu item or just remove it if the count is gone.  If
    that's the case we can update time. */
 static void
-count_cb (IndicateListener * listener, IndicateListenerServer * server, IndicateListenerIndicator * indicator, gchar * property, gchar * propertydata, gpointer data)
+count_cb (IndicateListener * listener, IndicateListenerServer * server, IndicateListenerIndicator * indicator, gchar * property, const gchar * propertydata, gpointer data)
 {
 	g_debug("Got Count Information");
 	ImMenuItem * self = IM_MENU_ITEM(data);
@@ -349,7 +358,7 @@ count_cb (IndicateListener * listener, IndicateListenerServer * server, Indicate
    this indicator should be calling for attention or not.  If we are,
    we need to signal that. */
 static void
-attention_cb (IndicateListener * listener, IndicateListenerServer * server, IndicateListenerIndicator * indicator, gchar * property, gchar * propertydata, gpointer data)
+attention_cb (IndicateListener * listener, IndicateListenerServer * server, IndicateListenerIndicator * indicator, gchar * property, const GValue * propertydata, gpointer data)
 {
 	g_debug("Got Attention Information");
 	ImMenuItem * self = IM_MENU_ITEM(data);
@@ -364,10 +373,19 @@ attention_cb (IndicateListener * listener, IndicateListenerServer * server, Indi
 	ImMenuItemPrivate * priv = IM_MENU_ITEM_GET_PRIVATE(self);
 
 	gboolean wantit;
-	if (propertydata == NULL || propertydata[0] == '\0' || !g_strcmp0(propertydata, "false")) {
-		wantit = FALSE;
+	if (G_VALUE_HOLDS_BOOLEAN(propertydata)) {
+		wantit = g_value_get_boolean(propertydata);
+	} else if (G_VALUE_HOLDS_STRING(propertydata)) {
+		const gchar * propstring = g_value_get_string(propertydata);
+
+		if (propstring == NULL || propstring[0] == '\0' || !g_strcmp0(propstring, "false")) {
+			wantit = FALSE;
+		} else {
+			wantit = TRUE;
+		}
 	} else {
-		wantit = TRUE;
+		g_warning("Got property '%s' of an unknown type.", property);
+		return;
 	}
 
 	if (priv->attention != wantit) {
@@ -380,11 +398,11 @@ attention_cb (IndicateListener * listener, IndicateListenerServer * server, Indi
 
 /* Callback when the item gets clicked on from the Messaging Menu */
 static void
-activate_cb (ImMenuItem * self, gpointer data)
+activate_cb (ImMenuItem * self, guint timestamp, gpointer data)
 {
 	ImMenuItemPrivate * priv = IM_MENU_ITEM_GET_PRIVATE(self);
 
-	indicate_listener_display(priv->listener, priv->server, priv->indicator);
+	indicate_listener_display(priv->listener, priv->server, priv->indicator, timestamp);
 }
 
 /* Callback when a property gets modified.  It figures out which one
@@ -409,7 +427,7 @@ indicator_modified_cb (IndicateListener * listener, IndicateListenerServer * ser
 	} else if (!g_strcmp0(property, INDICATE_INDICATOR_MESSAGES_PROP_COUNT)) {
 		indicate_listener_get_property(listener, server, indicator, INDICATE_INDICATOR_MESSAGES_PROP_COUNT, count_cb, self);	
 	} else if (!g_strcmp0(property, INDICATE_INDICATOR_MESSAGES_PROP_ATTENTION)) {
-		indicate_listener_get_property(listener, server, indicator, INDICATE_INDICATOR_MESSAGES_PROP_ATTENTION, attention_cb, self);	
+		indicate_listener_get_property_value(listener, server, indicator, INDICATE_INDICATOR_MESSAGES_PROP_ATTENTION, attention_cb, self);	
 	} else if (!g_strcmp0(property, "sender")) {
 		/* This is a compatibility string with v1 and should be removed */
 		g_debug("Indicator is using 'sender' property which is a v1 string.");
@@ -434,7 +452,7 @@ im_menu_item_new (IndicateListener * listener, IndicateListenerServer * server,
 	priv->attention = FALSE;
 	priv->show = TRUE;
 
-	dbusmenu_menuitem_property_set(DBUSMENU_MENUITEM(self), "type", INDICATOR_MENUITEM_TYPE);
+	dbusmenu_menuitem_property_set(DBUSMENU_MENUITEM(self), DBUSMENU_MENUITEM_PROP_TYPE, INDICATOR_MENUITEM_TYPE);
 
 	indicate_listener_displayed(listener, server, indicator, TRUE);
 
@@ -442,7 +460,7 @@ im_menu_item_new (IndicateListener * listener, IndicateListenerServer * server,
 	indicate_listener_get_property_time(listener, server, indicator, INDICATE_INDICATOR_MESSAGES_PROP_TIME, time_cb, self);	
 	indicate_listener_get_property(listener, server, indicator, INDICATE_INDICATOR_MESSAGES_PROP_ICON, icon_cb, self);	
 	indicate_listener_get_property(listener, server, indicator, INDICATE_INDICATOR_MESSAGES_PROP_COUNT, count_cb, self);	
-	indicate_listener_get_property(listener, server, indicator, INDICATE_INDICATOR_MESSAGES_PROP_ATTENTION, attention_cb, self);	
+	indicate_listener_get_property_value(listener, server, indicator, INDICATE_INDICATOR_MESSAGES_PROP_ATTENTION, attention_cb, self);	
 	indicate_listener_get_property(listener, server, indicator, "sender", sender_cb, self);	
 
 	g_signal_connect(G_OBJECT(self), DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED, G_CALLBACK(activate_cb), NULL);
