@@ -55,7 +55,8 @@ static MessageServiceDbus * dbus_interface = NULL;
 #define DESKTOP_FILE_GROUP        "Messaging Menu"
 #define DESKTOP_FILE_KEY_DESKTOP  "DesktopFile"
 
-static void server_shortcuts_changed (AppMenuItem * appitem, gpointer data);
+static void server_shortcut_added (AppMenuItem * appitem, DbusmenuMenuitem * mi, gpointer data);
+static void server_shortcut_removed (AppMenuItem * appitem, DbusmenuMenuitem * mi, gpointer data);
 static void server_count_changed (AppMenuItem * appitem, guint count, gpointer data);
 static void server_name_changed (AppMenuItem * appitem, gchar * name, gpointer data);
 static void im_time_changed (ImMenuItem * imitem, glong seconds, gpointer data);
@@ -314,7 +315,7 @@ desktop_file_from_keyfile (const gchar * definition_file)
 }
 
 /* Add a definition file into the black list and eclipse
-   and launchers that have the same file. */
+   any launchers that have the same file. */
 static gboolean
 blacklist_add (gpointer udata)
 {
@@ -358,7 +359,7 @@ blacklist_add_core (gchar * desktop, gchar * definition)
 	}
 
 	/* Actually blacklist this thing */
-	g_hash_table_insert(blacklist, desktop, definition);
+	g_hash_table_insert(blacklist, g_strdup(desktop), g_strdup(definition));
 	g_debug("Adding Blacklist item '%s' for desktop '%s'", definition, desktop);
 
 	/* Go through and eclipse folks */
@@ -582,7 +583,8 @@ server_added (IndicateListener * listener, IndicateListenerServer * server, gcha
 	/* Connect the signals up to the menu item */
 	g_signal_connect(G_OBJECT(menuitem), APP_MENU_ITEM_SIGNAL_COUNT_CHANGED, G_CALLBACK(server_count_changed), sl_item);
 	g_signal_connect(G_OBJECT(menuitem), APP_MENU_ITEM_SIGNAL_NAME_CHANGED,  G_CALLBACK(server_name_changed),  menushell);
-	g_signal_connect(G_OBJECT(menuitem), APP_MENU_ITEM_SIGNAL_SHORTCUTS_CHANGED,  G_CALLBACK(server_shortcuts_changed),  menushell);
+	g_signal_connect(G_OBJECT(menuitem), APP_MENU_ITEM_SIGNAL_SHORTCUT_ADDED,  G_CALLBACK(server_shortcut_added),  menushell);
+	g_signal_connect(G_OBJECT(menuitem), APP_MENU_ITEM_SIGNAL_SHORTCUT_REMOVED,  G_CALLBACK(server_shortcut_removed),  menushell);
 
 	/* Put our new menu item in, with the separator behind it.
 	   resort_menu will take care of whether it should be hidden
@@ -605,47 +607,26 @@ server_added (IndicateListener * listener, IndicateListenerServer * server, gcha
 	return;
 }
 
-/* The shortcuts have changed, let's just remove them and put
-   the back. */
+/* Server shortcut has been added */
 static void
-server_shortcuts_changed (AppMenuItem * appitem, gpointer data)
+server_shortcut_added (AppMenuItem * appitem, DbusmenuMenuitem * mi, gpointer data)
 {
-	g_debug("Application Shortcuts changed");
+	g_debug("Application Shortcut added: %s", mi != NULL ? dbusmenu_menuitem_property_get(mi, DBUSMENU_MENUITEM_PROP_LABEL) : "none");
 	DbusmenuMenuitem * shell = DBUSMENU_MENUITEM(data);
-	gboolean appitemfound = FALSE;
-	GList * children = dbusmenu_menuitem_get_children(shell);
-	GList * removelist = NULL;
-
-	while (children != NULL) {
-		if (!appitemfound && children->data != appitem) {
-			children = g_list_next(children);
-			continue;
-		}
-		appitemfound = TRUE;
-
-		if (!DBUSMENU_IS_MENUITEM_PROXY(children->data)) {
-			break;
-		}
-
-		removelist = g_list_prepend(removelist, children->data);
-	}
-
-	GList * removeitem;
-	for (removeitem = removelist; removeitem != NULL; removeitem = g_list_next(removeitem)) {
-		dbusmenu_menuitem_child_delete(shell, DBUSMENU_MENUITEM(removeitem->data));
-	}
-	g_list_free(removeitem);
-
-	GList * shortcuts = app_menu_item_get_items(appitem);
-	while (shortcuts != NULL) {
-		DbusmenuMenuitem * mi = DBUSMENU_MENUITEM(shortcuts->data);
-		g_debug("\tAdding shortcut: %s", dbusmenu_menuitem_property_get(mi, DBUSMENU_MENUITEM_PROP_LABEL));
+	if (mi != NULL) {
 		dbusmenu_menuitem_child_append(shell, mi);
-		shortcuts = g_list_next(shortcuts);
 	}
-
 	resort_menu(shell);
+	return;
+}
 
+/* Server shortcut has been removed */
+static void
+server_shortcut_removed (AppMenuItem * appitem, DbusmenuMenuitem * mi, gpointer data)
+{
+	g_debug("Application Shortcut removed: %s", mi != NULL ? dbusmenu_menuitem_property_get(mi, DBUSMENU_MENUITEM_PROP_LABEL) : "none");
+	DbusmenuMenuitem * shell = DBUSMENU_MENUITEM(data);
+	dbusmenu_menuitem_child_delete(shell, mi);
 	return;
 }
 
@@ -756,16 +737,31 @@ server_removed (IndicateListener * listener, IndicateListenerServer * server, gc
 
 	/* If there is a menu item, let's get rid of it. */
 	if (sltp->menuitem != NULL) {
-		dbusmenu_menuitem_property_set_bool(DBUSMENU_MENUITEM(sltp->menuitem), DBUSMENU_MENUITEM_PROP_VISIBLE, TRUE);
+		/* If there are shortcuts remove them */
+		GList * shortcuts = app_menu_item_get_items(sltp->menuitem);
+		while (shortcuts != NULL) {
+			g_debug("\tRemoving shortcut: %s", dbusmenu_menuitem_property_get(DBUSMENU_MENUITEM(shortcuts->data), DBUSMENU_MENUITEM_PROP_LABEL));
+			dbusmenu_menuitem_property_set_bool(DBUSMENU_MENUITEM(shortcuts->data), DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
+			dbusmenu_menuitem_child_delete(DBUSMENU_MENUITEM(data), DBUSMENU_MENUITEM(shortcuts->data));
+			shortcuts = g_list_next(shortcuts);
+		}
+
+		g_debug("\tRemoving item");
+		dbusmenu_menuitem_property_set_bool(DBUSMENU_MENUITEM(sltp->menuitem), DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
 		dbusmenu_menuitem_child_delete(DBUSMENU_MENUITEM(data), DBUSMENU_MENUITEM(sltp->menuitem));
 		g_object_unref(G_OBJECT(sltp->menuitem));
+	} else {
+		g_debug("\tNo menuitem");
 	}
-
+	
 	/* If there is a separator, let's get rid of it. */
 	if (sltp->separator != NULL) {
+		g_debug("\tRemoving separator");
 		dbusmenu_menuitem_property_set_bool(DBUSMENU_MENUITEM(sltp->separator), DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
 		dbusmenu_menuitem_child_delete(DBUSMENU_MENUITEM(data), DBUSMENU_MENUITEM(sltp->separator));
 		g_object_unref(G_OBJECT(sltp->separator));
+	} else {
+		g_debug("\tNo separator");
 	}
 
 	if (sltp->attention) {
@@ -1263,6 +1259,14 @@ destroy_launcher (gpointer data)
 	g_list_free(li->appdiritems);
 
 	if (li->menuitem != NULL) {
+		/* If there are shortcuts remove them */
+		GList * shortcuts = launcher_menu_item_get_items(li->menuitem);
+		while (shortcuts != NULL) {
+			dbusmenu_menuitem_property_set_bool(DBUSMENU_MENUITEM(shortcuts->data), DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
+			dbusmenu_menuitem_child_delete(DBUSMENU_MENUITEM(data), DBUSMENU_MENUITEM(shortcuts->data));
+			shortcuts = g_list_next(shortcuts);
+		}
+
 		dbusmenu_menuitem_property_set_bool(DBUSMENU_MENUITEM(li->menuitem), DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
 		dbusmenu_menuitem_child_delete(root_menuitem, DBUSMENU_MENUITEM(li->menuitem));
 		g_object_unref(G_OBJECT(li->menuitem));
@@ -1358,6 +1362,14 @@ build_launcher_core (const gchar * desktop)
 			dbusmenu_menuitem_property_set_bool(ll->separator, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
 		}
 
+		/* Check to see if any of the current applications should
+		   be eclipsing us. */
+		GList * server;
+		for (server = serverList; server != NULL; server = g_list_next(server)) {
+			serverList_t * slt = (serverList_t *)server->data;
+			check_eclipses(slt->menuitem);
+		}
+
 		resort_menu(root_menuitem);
 		check_hidden();
 	} else {
